;;; The irc reply and error codes as specified by RFC 2812.

;;;
;;;  Copyright (C) The Internet Society (2000).  All Rights Reserved.
;;;
;;;   This document and translations of it may be copied and furnished to
;;;   others, and derivative works that comment on or otherwise explain it
;;;   or assist in its implementation may be prepared, copied, published
;;;   and distributed, in whole or in part, without restriction of any
;;;   kind, provided that the above copyright notice and this paragraph are
;;;   included on all such copies and derivative works.  However, this
;;;   document itself may not be modified in any way, such as by removing
;;;   the copyright notice or references to the Internet Society or other
;;;   Internet organizations, except as needed for the purpose of
;;;   developing Internet standards in which case the procedures for
;;;   copyrights defined in the Internet Standards process must be
;;;   followed, or as required to translate it into languages other than
;;;   English.
;;;
;;;   The limited permissions granted above are perpetual and will not be
;;;   revoked by the Internet Society or its successors or assigns.
;;;
;;;   This document and the information contained herein is provided on an
;;;   "AS IS" basis and THE INTERNET SOCIETY AND THE INTERNET ENGINEERING
;;;   TASK FORCE DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING
;;;   BUT NOT LIMITED TO ANY WARRANTY THAT THE USE OF THE INFORMATION
;;;   HEREIN WILL NOT INFRINGE ANY RIGHTS OR ANY IMPLIED WARRANTIES OF
;;;   MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.

(define-module (rfc-2812))

;;   Numerics in the range from 001 to 099 are used for client-server
;;   connections only and should never travel between servers.  Replies
;;   generated in the response to commands are found in the range from 200
;;   to 399.
;;
;;   Error replies are found in the range from 400 to 599.


;;
;; Command responses
;;

(define-public RPL_WELCOME "001")

;;  001    RPL_WELCOME
;;               "Welcome to the Internet Relay Network
;;                <nick>!<user>@<host>"

(define-public RPL_YOURHOST "002")

;;        002    RPL_YOURHOST
;;               "Your host is <servername>, running version <ver>"

(define-public RPL_CREATED "003")

;;        003    RPL_CREATED
;;               "This server was created <date>"

(define-public RPL_MYINFO "004")

;;        004    RPL_MYINFO
;;               "<servername> <version> <available user modes>
;; <available channel modes>"
;; 
;;          - The server sends Replies 001 to 004 to a user upon
;;            successful registration.
;; 

(define-public RPL_BOUNCE "005")

;;        005    RPL_BOUNCE
;;               "Try server <server name>, port <port number>"
;; 
;;          - Sent by the server to a user to suggest an alternative
;;            server.  This is often used when the connection is
;;            refused because the server is already full.
;; 

(define-public RPL_USERHOST "302")

;;        302    RPL_USERHOST
;;               ":*1<reply> *( " " <reply> )"
;; 
;;          - Reply format used by USERHOST to list replies to
;;            the query list.  The reply string is composed as
;;            follows:
;; 
;;            reply = nickname [ "*" ] "=" ( "+" / "-" ) hostname
;; 
;;            The '*' indicates whether the client has registered
;;            as an Operator.  The '-' or '+' characters represent
;;            whether the client has set an AWAY message or not
;;            respectively.
;; 

(define-public RPL_ISON "303")

;;        303    RPL_ISON
;;               ":*1<nick> *( " " <nick> )"
;; 
;;          - Reply format used by ISON to list replies to the
;;            query list.
;; 

(define-public RPL_AWAY "301")

;;        301    RPL_AWAY
;;               "<nick> :<away message>"

(define-public RPL_UNAWAY "305")

;;        305    RPL_UNAWAY
;;               ":You are no longer marked as being away"

(define-public RPL_NOWAWAY "306")

;;        306    RPL_NOWAWAY
;;               ":You have been marked as being away"
;; 
;;          - These replies are used with the AWAY command (if
;;            allowed).  RPL_AWAY is sent to any client sending a
;;            PRIVMSG to a client which is away.  RPL_AWAY is only
;;            sent by the server to which the client is connected.
;;            Replies RPL_UNAWAY and RPL_NOWAWAY are sent when the
;;            client removes and sets an AWAY message.
;; 

(define-public RPL_WHOISUSER "311")

;;        311    RPL_WHOISUSER
;;               "<nick> <user> <host> * :<real name>"

(define-public RPL_WHOISSERVER "312")

;;        312    RPL_WHOISSERVER
;;               "<nick> <server> :<server info>"

(define-public RPL_WHOISOPERATOR "313")

;;        313    RPL_WHOISOPERATOR
;;               "<nick> :is an IRC operator"
;; 
;; 

(define-public RPL_WHOISIDLE "317")

;;        317    RPL_WHOISIDLE
;;               "<nick> <integer> :seconds idle"

(define-public RPL_ENDOFWHOIS "318")

;;        318    RPL_ENDOFWHOIS
;;               "<nick> :End of WHOIS list"

(define-public RPL_WHOISCHANNELS "319")

;;        319    RPL_WHOISCHANNELS
;;               "<nick> :*( ( "@" / "+" ) <channel> " " )"
;; 
;;          - Replies 311 - 313, 317 - 319 are all replies
;;            generated in response to a WHOIS message.  Given that
;;            there are enough parameters present, the answering
;;            server MUST either formulate a reply out of the above
;;            numerics (if the query nick is found) or return an
;;            error reply.  The '*' in RPL_WHOISUSER is there as
;;            the literal character and not as a wild card.  For
;;            each reply set, only RPL_WHOISCHANNELS may appear
;;            more than once (for long lists of channel names).
;;            The '@' and '+' characters next to the channel name
;;            indicate whether a client is a channel operator or
;;            has been granted permission to speak on a moderated
;;            channel.  The RPL_ENDOFWHOIS reply is used to mark
;;            the end of processing a WHOIS message.
;; 

(define-public RPL_WHOWASUSER "314")

;;        314    RPL_WHOWASUSER
;;               "<nick> <user> <host> * :<real name>"

(define-public RPL_ENDOFWHOWAS "369")

;;        369    RPL_ENDOFWHOWAS
;;               "<nick> :End of WHOWAS"
;; 
;;          - When replying to a WHOWAS message, a server MUST use
;;            the replies RPL_WHOWASUSER, RPL_WHOISSERVER or
;;            ERR_WASNOSUCHNICK for each nickname in the presented
;;            list.  At the end of all reply batches, there MUST
;;            be RPL_ENDOFWHOWAS (even if there was only one reply
;;            and it was an error).
;; 

(define-public RPL_LISTSTART "321")

;;        321    RPL_LISTSTART
;;               Obsolete. Not used.
;; 

(define-public RPL_LIST "322")

;;        322    RPL_LIST
;;               "<channel> <# visible> :<topic>"

(define-public RPL_LISTEND "323")

;;        323    RPL_LISTEND
;;               ":End of LIST"
;; 
;;          - Replies RPL_LIST, RPL_LISTEND mark the actual replies
;;            with data and end of the server's response to a LIST
;;            command.  If there are no channels available to return,
;;            only the end reply MUST be sent.
;; 

(define-public RPL_UNIQOPIS "325")

;;        325    RPL_UNIQOPIS
;;               "<channel> <nickname>"
;; 

(define-public RPL_CHANNELMODEIS "324")

;;        324    RPL_CHANNELMODEIS
;;               "<channel> <mode> <mode params>"
;; 

(define-public RPL_NOTOPIC "331")

;;        331    RPL_NOTOPIC
;;               "<channel> :No topic is set"

(define-public RPL_TOPIC "332")

;;        332    RPL_TOPIC
;;               "<channel> :<topic>"
;; 
;;          - When sending a TOPIC message to determine the
;;            channel topic, one of two replies is sent.  If
;;            the topic is set, RPL_TOPIC is sent back else
;;            RPL_NOTOPIC.
;; 

(define-public RPL_INVITING "341")

;;        341    RPL_INVITING
;;               "<channel> <nick>"
;; 
;;          - Returned by the server to indicate that the
;;            attempted INVITE message was successful and is
;;            being passed onto the end client.
;; 

(define-public RPL_SUMMONING "342")

;;        342    RPL_SUMMONING
;;               "<user> :Summoning user to IRC"
;; 
;;          - Returned by a server answering a SUMMON message to
;;            indicate that it is summoning that user.
;; 

(define-public RPL_INVITELIST "346")

;;        346    RPL_INVITELIST
;;               "<channel> <invitemask>"

(define-public RPL_ENDOFINVITELIST "347")

;;        347    RPL_ENDOFINVITELIST
;;               "<channel> :End of channel invite list"
;; 
;;          - When listing the 'invitations masks' for a given channel,
;;            a server is required to send the list back using the
;;            RPL_INVITELIST and RPL_ENDOFINVITELIST messages.  A
;;            separate RPL_INVITELIST is sent for each active mask.
;;            After the masks have been listed (or if none present) a
;;            RPL_ENDOFINVITELIST MUST be sent.
;; 

(define-public RPL_EXCEPTLIST "348")

;;        348    RPL_EXCEPTLIST
;;               "<channel> <exceptionmask>"

(define-public RPL_ENDOFEXCEPTLIST "349")

;;        349    RPL_ENDOFEXCEPTLIST
;;               "<channel> :End of channel exception list"
;; 
;;          - When listing the 'exception masks' for a given channel,
;;            a server is required to send the list back using the
;;            RPL_EXCEPTLIST and RPL_ENDOFEXCEPTLIST messages.  A
;;            separate RPL_EXCEPTLIST is sent for each active mask.
;;            After the masks have been listed (or if none present)
;;            a RPL_ENDOFEXCEPTLIST MUST be sent.
;; 

(define-public RPL_VERSION "351")

;;        351    RPL_VERSION
;;               "<version>.<debuglevel> <server> :<comments>"
;; 
;;          - Reply by the server showing its version details.
;;            The <version> is the version of the software being
;;            used (including any patchlevel revisions) and the
;;            <debuglevel> is used to indicate if the server is
;;            running in "debug mode".
;; 
;;            The "comments" field may contain any comments about
;;            the version or further version details.
;; 

(define-public RPL_WHOREPLY "352")

;;        352    RPL_WHOREPLY
;;               "<channel> <user> <host> <server> <nick>
;; ( "H" / "G" > ["*"] [ ( "@" / "+" ) ]
;;               :<hopcount> <real name>"
;; 

(define-public RPL_ENDOFWHO "315")

;;        315    RPL_ENDOFWHO
;;               "<name> :End of WHO list"
;; 
;;          - The RPL_WHOREPLY and RPL_ENDOFWHO pair are used
;;            to answer a WHO message.  The RPL_WHOREPLY is only
;;            sent if there is an appropriate match to the WHO
;;            query.  If there is a list of parameters supplied
;;            with a WHO message, a RPL_ENDOFWHO MUST be sent
;;            after processing each list item with <name> being
;;            the item.
;; 

(define-public RPL_NAMREPLY "353")

;;        353    RPL_NAMREPLY
;;               "( "=" / "*" / "@" ) <channel>
;; :[ "@" / "+" ] <nick> *( " " [ "@" / "+" ] <nick> )
;;          - "@" is used for secret channels, "*" for private
;;            channels, and "=" for others (public channels).
;; 

(define-public RPL_ENDOFNAMES "366")

;;        366    RPL_ENDOFNAMES
;;               "<channel> :End of NAMES list"
;; 
;;          - To reply to a NAMES message, a reply pair consisting
;;            of RPL_NAMREPLY and RPL_ENDOFNAMES is sent by the
;;            server back to the client.  If there is no channel
;;            found as in the query, then only RPL_ENDOFNAMES is
;;            returned.  The exception to this is when a NAMES
;;            message is sent with no parameters and all visible
;;            channels and contents are sent back in a series of
;;            RPL_NAMEREPLY messages with a RPL_ENDOFNAMES to mark
;;            the end.
;; 

(define-public RPL_LINKS "364")

;;        364    RPL_LINKS
;;               "<mask> <server> :<hopcount> <server info>"

(define-public RPL_ENDOFLINKS "365")

;;        365    RPL_ENDOFLINKS
;;               "<mask> :End of LINKS list"
;; 
;;          - In replying to the LINKS message, a server MUST send
;;            replies back using the RPL_LINKS numeric and mark the
;;            end of the list using an RPL_ENDOFLINKS reply.
;; 

(define-public RPL_BANLIST "367")

;;        367    RPL_BANLIST
;;               "<channel> <banmask>"

(define-public RPL_ENDOFBANLIST "368")

;;        368    RPL_ENDOFBANLIST
;;               "<channel> :End of channel ban list"
;; 
;;          - When listing the active 'bans' for a given channel,
;;            a server is required to send the list back using the
;;            RPL_BANLIST and RPL_ENDOFBANLIST messages.  A separate
;;            RPL_BANLIST is sent for each active banmask.  After the
;;            banmasks have been listed (or if none present) a
;;            RPL_ENDOFBANLIST MUST be sent.
;; 

(define-public RPL_INFO "371")

;;        371    RPL_INFO
;;               ":<string>"

(define-public RPL_ENDOFINFO "374")

;;        374    RPL_ENDOFINFO
;;               ":End of INFO list"
;; 
;;          - A server responding to an INFO message is required to
;;            send all its 'info' in a series of RPL_INFO messages
;;            with a RPL_ENDOFINFO reply to indicate the end of the
;;            replies.
;; 

(define-public RPL_MOTDSTART "375")

;;        375    RPL_MOTDSTART
;;               ":- <server> Message of the day - "

(define-public RPL_MOTD "372")

;;        372    RPL_MOTD
;;               ":- <text>"

(define-public RPL_ENDOFMOTD "376")

;;        376    RPL_ENDOFMOTD
;;               ":End of MOTD command"
;; 
;;          - When responding to the MOTD message and the MOTD file
;;            is found, the file is displayed line by line, with
;;            each line no longer than 80 characters, using
;;            RPL_MOTD format replies.  These MUST be surrounded
;;            by a RPL_MOTDSTART (before the RPL_MOTDs) and an
;;            RPL_ENDOFMOTD (after).
;; 

(define-public RPL_YOUREOPER "381")

;;        381    RPL_YOUREOPER
;;               ":You are now an IRC operator"
;; 
;;          - RPL_YOUREOPER is sent back to a client which has
;;            just successfully issued an OPER message and gained
;;            operator status.
;; 

(define-public RPL_REHASHING "382")

;;        382    RPL_REHASHING
;;               "<config file> :Rehashing"
;; 
;;          - If the REHASH option is used and an operator sends
;;            a REHASH message, an RPL_REHASHING is sent back to
;;            the operator.
;; 

(define-public RPL_YOURESERVICE "383")

;;        383    RPL_YOURESERVICE
;;               "You are service <servicename>"
;; 
;;          - Sent by the server to a service upon successful
;;            registration.
;; 

(define-public RPL_TIME "391")

;;        391    RPL_TIME
;;               "<server> :<string showing server's local time>"
;; 
;;          - When replying to the TIME message, a server MUST send
;;            the reply using the RPL_TIME format above.  The string
;;            showing the time need only contain the correct day and
;;            time there.  There is no further requirement for the
;;            time string.
;; 

(define-public RPL_USERSSTART "392")

;;        392    RPL_USERSSTART
;;               ":UserID   Terminal  Host"

(define-public RPL_USERS "393")

;;        393    RPL_USERS
;;               ":<username> <ttyline> <hostname>"

(define-public RPL_ENDOFUSERS "394")

;;        394    RPL_ENDOFUSERS
;;               ":End of users"

(define-public RPL_NOUSERS "395")

;;        395    RPL_NOUSERS
;;               ":Nobody logged in"
;; 
;;          - If the USERS message is handled by a server, the
;;            replies RPL_USERSTART, RPL_USERS, RPL_ENDOFUSERS and
;;            RPL_NOUSERS are used.  RPL_USERSSTART MUST be sent
;;            first, following by either a sequence of RPL_USERS
;;            or a single RPL_NOUSER.  Following this is
;;            RPL_ENDOFUSERS.
;; 
;; 

(define-public RPL_TRACELINK "200")

;;        200    RPL_TRACELINK
;;               "Link <version & debug level> <destination>
;; <next server> V<protocol version>
;; <link uptime in seconds> <backstream sendq>
;; <upstream sendq>"

(define-public RPL_TRACECONNECTING "201")

;;        201    RPL_TRACECONNECTING
;;               "Try. <class> <server>"

(define-public RPL_TRACEHANDSHAKE "202")

;;        202    RPL_TRACEHANDSHAKE
;;               "H.S. <class> <server>"

(define-public RPL_TRACEUNKNOWN "203")

;;        203    RPL_TRACEUNKNOWN
;;               "???? <class> [<client IP address in dot form>]"

(define-public RPL_TRACEOPERATOR "204")

;;        204    RPL_TRACEOPERATOR
;;               "Oper <class> <nick>"

(define-public RPL_TRACEUSER "205")

;;        205    RPL_TRACEUSER
;;               "User <class> <nick>"

(define-public RPL_TRACESERVER "206")

;;        206    RPL_TRACESERVER
;;               "Serv <class> <int>S <int>C <server>
;; <nick!user|*!*>@<host|server> V<protocol version>"

(define-public RPL_TRACESERVICE "207")

;;        207    RPL_TRACESERVICE
;;               "Service <class> <name> <type> <active type>"

(define-public RPL_TRACENEWTYPE "208")

;;        208    RPL_TRACENEWTYPE
;;               "<newtype> 0 <client name>"

(define-public RPL_TRACECLASS "209")

;;        209    RPL_TRACECLASS
;;               "Class <class> <count>"

(define-public RPL_TRACERECONNECT "210")

;;        210    RPL_TRACERECONNECT
;;               Unused.

(define-public RPL_TRACELOG "261")

;;        261    RPL_TRACELOG
;;               "File <logfile> <debug level>"

(define-public RPL_TRACEEND "262")

;;        262    RPL_TRACEEND
;;               "<server name> <version & debug level> :End of TRACE"
;; 
;;          - The RPL_TRACE* are all returned by the server in
;;            response to the TRACE message.  How many are
;;            returned is dependent on the TRACE message and
;;            whether it was sent by an operator or not.  There
;;            is no predefined order for which occurs first.
;;            Replies RPL_TRACEUNKNOWN, RPL_TRACECONNECTING and
;;            RPL_TRACEHANDSHAKE are all used for connections
;;            which have not been fully established and are either
;;            unknown, still attempting to connect or in the
;;            process of completing the 'server handshake'.
;;            RPL_TRACELINK is sent by any server which handles
;;            a TRACE message and has to pass it on to another
;;            server.  The list of RPL_TRACELINKs sent in
;;            response to a TRACE command traversing the IRC
;;            network should reflect the actual connectivity of
;;            the servers themselves along that path.
;;            RPL_TRACENEWTYPE is to be used for any connection
;;            which does not fit in the other categories but is
;;            being displayed anyway.
;;            RPL_TRACEEND is sent to indicate the end of the list.
;; 

(define-public RPL_STATSLINKINFO "211")

;;        211    RPL_STATSLINKINFO
;;               "<linkname> <sendq> <sent messages>
;; <sent Kbytes> <received messages>
;; <received Kbytes> <time open>"
;; 
;;          - reports statistics on a connection.  <linkname>
;; identifies the particular connection, <sendq> is
;;            the amount of data that is queued and waiting to be
;;            sent <sent messages> the number of messages sent,
;;            and <sent Kbytes> the amount of data sent, in
;;            Kbytes. <received messages> and <received Kbytes>
;; are the equivalent of <sent messages> and <sent
;;            Kbytes> for received data, respectively.  <time
;;            open> indicates how long ago the connection was
;;            opened, in seconds.
;; 

(define-public RPL_STATSCOMMANDS "212")

;;        212    RPL_STATSCOMMANDS
;;               "<command> <count> <byte count> <remote count>"
;; 
;;          - reports statistics on commands usage.
;; 

(define-public RPL_ENDOFSTATS "219")

;;        219    RPL_ENDOFSTATS
;;               "<stats letter> :End of STATS report"
;; 

(define-public RPL_STATSUPTIME "242")

;;        242    RPL_STATSUPTIME
;;               ":Server Up %d days %d:%02d:%02d"
;; 
;;          - reports the server uptime.
;; 

(define-public RPL_STATSOLINE "243")

;;        243    RPL_STATSOLINE
;;               "O <hostmask> * <name>"
;; 
;;          - reports the allowed hosts from where user may become IRC
;;            operators.
;; 

(define-public RPL_UMODEIS "221")

;;        221    RPL_UMODEIS
;;               "<user mode string>"
;; 
;;          - To answer a query about a client's own mode,
;;            RPL_UMODEIS is sent back.
;; 

(define-public RPL_SERVLIST "234")

;;        234    RPL_SERVLIST
;;               "<name> <server> <mask> <type> <hopcount> <info>"
;; 

(define-public RPL_SERVLISTEND "235")

;;        235    RPL_SERVLISTEND
;;               "<mask> <type> :End of service listing"
;; 
;;          - When listing services in reply to a SERVLIST message,
;;            a server is required to send the list back using the
;;            RPL_SERVLIST and RPL_SERVLISTEND messages.  A separate
;;            RPL_SERVLIST is sent for each service.  After the
;;            services have been listed (or if none present) a
;;            RPL_SERVLISTEND MUST be sent.
;; 

(define-public RPL_LUSERCLIENT "251")

;;        251    RPL_LUSERCLIENT
;;               ":There are <integer> users and <integer>
;; services on <integer> servers"

(define-public RPL_LUSEROP "252")

;;        252    RPL_LUSEROP
;;               "<integer> :operator(s) online"

(define-public RPL_LUSERUNKNOWN "253")

;;        253    RPL_LUSERUNKNOWN
;;               "<integer> :unknown connection(s)"

(define-public RPL_LUSERCHANNELS "254")

;;        254    RPL_LUSERCHANNELS
;;               "<integer> :channels formed"

(define-public RPL_LUSERME "255")

;;        255    RPL_LUSERME
;;               ":I have <integer> clients and <integer>
;; servers"
;; 
;;          - In processing an LUSERS message, the server
;;            sends a set of replies from RPL_LUSERCLIENT,
;;            RPL_LUSEROP, RPL_USERUNKNOWN,
;;            RPL_LUSERCHANNELS and RPL_LUSERME.  When
;;            replying, a server MUST send back
;;            RPL_LUSERCLIENT and RPL_LUSERME.  The other
;;            replies are only sent back if a non-zero count
;;            is found for them.
;; 

(define-public RPL_ADMINME "256")

;;        256    RPL_ADMINME
;;               "<server> :Administrative info"

(define-public RPL_ADMINLOC1 "257")

;;        257    RPL_ADMINLOC1
;;               ":<admin info>"

(define-public RPL_ADMINLOC2 "258")

;;        258    RPL_ADMINLOC2
;;               ":<admin info>"

(define-public RPL_ADMINEMAIL "259")

;;        259    RPL_ADMINEMAIL
;;               ":<admin info>"
;; 
;;          - When replying to an ADMIN message, a server
;;            is expected to use replies RPL_ADMINME
;;            through to RPL_ADMINEMAIL and provide a text
;;            message with each.  For RPL_ADMINLOC1 a
;;            description of what city, state and country
;;            the server is in is expected, followed by
;;            details of the institution (RPL_ADMINLOC2)
;;            and finally the administrative contact for the
;;            server (an email address here is REQUIRED)
;;            in RPL_ADMINEMAIL.
;; 

(define-public RPL_TRYAGAIN "263")

;;        263    RPL_TRYAGAIN
;;               "<command> :Please wait a while and try again."
;; 
;;          - When a server drops a command without processing it,
;;            it MUST use the reply RPL_TRYAGAIN to inform the
;;            originating client.

;;
;; Error Replies
;;

(define-public ERR_NOSUCHNICK "401")

;;        401    ERR_NOSUCHNICK
;;               "<nickname> :No such nick/channel"
;; 
;;           - Used to indicate the nickname parameter supplied to a
;;             command is currently unused.
;; 

(define-public ERR_NOSUCHSERVER "402")

;;        402    ERR_NOSUCHSERVER
;;               "<server name> :No such server"
;; 
;;          - Used to indicate the server name given currently
;;            does not exist.
;; 

(define-public ERR_NOSUCHCHANNEL "403")

;;        403    ERR_NOSUCHCHANNEL
;;               "<channel name> :No such channel"
;; 
;;          - Used to indicate the given channel name is invalid.
;; 

(define-public ERR_CANNOTSENDTOCHAN "404")

;;        404    ERR_CANNOTSENDTOCHAN
;;               "<channel name> :Cannot send to channel"
;; 
;;          - Sent to a user who is either (a) not on a channel
;;            which is mode +n or (b) not a chanop (or mode +v) on
;;            a channel which has mode +m set or where the user is
;;            banned and is trying to send a PRIVMSG message to
;;            that channel.
;; 

(define-public ERR_TOOMANYCHANNELS "405")

;;        405    ERR_TOOMANYCHANNELS
;;               "<channel name> :You have joined too many channels"
;; 
;;          - Sent to a user when they have joined the maximum
;;            number of allowed channels and they try to join
;;            another channel.
;; 

(define-public ERR_WASNOSUCHNICK "406")

;;        406    ERR_WASNOSUCHNICK
;;               "<nickname> :There was no such nickname"
;; 
;;          - Returned by WHOWAS to indicate there is no history
;;            information for that nickname.
;; 

(define-public ERR_TOOMANYTARGETS "407")

;;        407    ERR_TOOMANYTARGETS
;;               "<target> :<error code> recipients. <abort message>"
;; 
;;          - Returned to a client which is attempting to send a
;;            PRIVMSG/NOTICE using the user@host destination format
;;            and for a user@host which has several occurrences.
;; 
;;          - Returned to a client which trying to send a
;;            PRIVMSG/NOTICE to too many recipients.
;; 
;;          - Returned to a client which is attempting to JOIN a safe
;;            channel using the shortname when there are more than one
;;            such channel.
;; 

(define-public ERR_NOSUCHSERVICE "408")

;;        408    ERR_NOSUCHSERVICE
;;               "<service name> :No such service"
;; 
;;          - Returned to a client which is attempting to send a SQUERY
;;            to a service which does not exist.
;; 

(define-public ERR_NOORIGIN "409")

;;        409    ERR_NOORIGIN
;;               ":No origin specified"
;; 
;;          - PING or PONG message missing the originator parameter.
;; 

(define-public ERR_NORECIPIENT "411")

;;        411    ERR_NORECIPIENT
;;               ":No recipient given (<command>)"

(define-public ERR_NOTEXTTOSEND "412")

;;        412    ERR_NOTEXTTOSEND
;;               ":No text to send"

(define-public ERR_NOTOPLEVEL "413")

;;        413    ERR_NOTOPLEVEL
;;               "<mask> :No toplevel domain specified"

(define-public ERR_WILDTOPLEVEL "414")

;;        414    ERR_WILDTOPLEVEL
;;               "<mask> :Wildcard in toplevel domain"

(define-public ERR_BADMASK "415")

;;        415    ERR_BADMASK
;;               "<mask> :Bad Server/host mask"
;; 
;;          - 412 - 415 are returned by PRIVMSG to indicate that
;;            the message wasn't delivered for some reason.
;;            ERR_NOTOPLEVEL and ERR_WILDTOPLEVEL are errors that
;;            are returned when an invalid use of
;;            "PRIVMSG $<server>" or "PRIVMSG #<host>" is attempted.
;; 

(define-public ERR_UNKNOWNCOMMAND "421")

;;        421    ERR_UNKNOWNCOMMAND
;;               "<command> :Unknown command"
;; 
;;          - Returned to a registered client to indicate that the
;;            command sent is unknown by the server.
;; 

(define-public ERR_NOMOTD "422")

;;        422    ERR_NOMOTD
;;               ":MOTD File is missing"
;; 
;;          - Server's MOTD file could not be opened by the server.
;; 

(define-public ERR_NOADMININFO "423")

;;        423    ERR_NOADMININFO
;;               "<server> :No administrative info available"
;; 
;;          - Returned by a server in response to an ADMIN message
;;            when there is an error in finding the appropriate
;;            information.
;; 

(define-public ERR_FILEERROR "424")

;;        424    ERR_FILEERROR
;;               ":File error doing <file op> on <file>"
;; 
;;          - Generic error message used to report a failed file
;;            operation during the processing of a message.
;; 

(define-public ERR_NONICKNAMEGIVEN "431")

;;        431    ERR_NONICKNAMEGIVEN
;;               ":No nickname given"
;; 
;;          - Returned when a nickname parameter expected for a
;;            command and isn't found.
;; 

(define-public ERR_ERRONEUSNICKNAME "432")

;;        432    ERR_ERRONEUSNICKNAME
;;               "<nick> :Erroneous nickname"
;; 
;;          - Returned after receiving a NICK message which contains
;;            characters which do not fall in the defined set.  See
;;            section 2.3.1 for details on valid nicknames.
;; 

(define-public ERR_NICKNAMEINUSE "433")

;;        433    ERR_NICKNAMEINUSE
;;               "<nick> :Nickname is already in use"
;; 
;;          - Returned when a NICK message is processed that results
;;            in an attempt to change to a currently existing
;;            nickname.
;; 

(define-public ERR_NICKCOLLISION "436")

;;        436    ERR_NICKCOLLISION
;;               "<nick> :Nickname collision KILL from <user>@<host>"
;; 
;;          - Returned by a server to a client when it detects a
;;            nickname collision (registered of a NICK that
;;            already exists by another server).
;; 

(define-public ERR_UNAVAILRESOURCE "437")

;;        437    ERR_UNAVAILRESOURCE
;;               "<nick/channel> :Nick/channel is temporarily unavailable"
;; 
;;          - Returned by a server to a user trying to join a channel
;;            currently blocked by the channel delay mechanism.
;; 
;;          - Returned by a server to a user trying to change nickname
;;            when the desired nickname is blocked by the nick delay
;;            mechanism.
;; 

(define-public ERR_USERNOTINCHANNEL "441")

;;        441    ERR_USERNOTINCHANNEL
;;               "<nick> <channel> :They aren't on that channel"
;; 
;;          - Returned by the server to indicate that the target
;;            user of the command is not on the given channel.
;; 

(define-public ERR_NOTONCHANNEL "442")

;;        442    ERR_NOTONCHANNEL
;;               "<channel> :You're not on that channel"
;; 
;;          - Returned by the server whenever a client tries to
;;            perform a channel affecting command for which the
;;            client isn't a member.
;; 

(define-public ERR_USERONCHANNEL "443")

;;        443    ERR_USERONCHANNEL
;;               "<user> <channel> :is already on channel"
;; 
;;          - Returned when a client tries to invite a user to a
;;            channel they are already on.
;; 

(define-public ERR_NOLOGIN "444")

;;        444    ERR_NOLOGIN
;;               "<user> :User not logged in"
;; 
;;          - Returned by the summon after a SUMMON command for a
;;            user was unable to be performed since they were not
;;            logged in.
;; 

(define-public ERR_SUMMONDISABLED "445")

;;        445    ERR_SUMMONDISABLED
;;               ":SUMMON has been disabled"
;; 
;;          - Returned as a response to the SUMMON command.  MUST be
;;            returned by any server which doesn't implement it.
;; 

(define-public ERR_USERSDISABLED "446")

;;        446    ERR_USERSDISABLED
;;               ":USERS has been disabled"
;; 
;;          - Returned as a response to the USERS command.  MUST be
;;            returned by any server which does not implement it.
;; 

(define-public ERR_NOTREGISTERED "451")

;;        451    ERR_NOTREGISTERED
;;               ":You have not registered"
;; 
;;          - Returned by the server to indicate that the client
;;            MUST be registered before the server will allow it
;;            to be parsed in detail.
;; 

(define-public ERR_NEEDMOREPARAMS "461")

;;        461    ERR_NEEDMOREPARAMS
;;               "<command> :Not enough parameters"
;; 
;;          - Returned by the server by numerous commands to
;;            indicate to the client that it didn't supply enough
;;            parameters.
;; 

(define-public ERR_ALREADYREGISTRED "462")

;;        462    ERR_ALREADYREGISTRED
;;               ":Unauthorized command (already registered)"
;; 
;;          - Returned by the server to any link which tries to
;;            change part of the registered details (such as
;;            password or user details from second USER message).
;; 

(define-public ERR_NOPERMFORHOST "463")

;;        463    ERR_NOPERMFORHOST
;;               ":Your host isn't among the privileged"
;; 
;;          - Returned to a client which attempts to register with
;;            a server which does not been setup to allow
;;            connections from the host the attempted connection
;;            is tried.
;; 

(define-public ERR_PASSWDMISMATCH "464")

;;        464    ERR_PASSWDMISMATCH
;;               ":Password incorrect"
;; 
;;          - Returned to indicate a failed attempt at registering
;;            a connection for which a password was required and
;;            was either not given or incorrect.
;; 

(define-public ERR_YOUREBANNEDCREEP "465")

;;        465    ERR_YOUREBANNEDCREEP
;;               ":You are banned from this server"
;; 
;;          - Returned after an attempt to connect and register
;;            yourself with a server which has been setup to
;;            explicitly deny connections to you.
;; 

(define-public ERR_YOUWILLBEBANNED "466")

;;        466    ERR_YOUWILLBEBANNED
;; 
;;          - Sent by a server to a user to inform that access to the
;;            server will soon be denied.
;; 

(define-public ERR_KEYSET "467")

;;        467    ERR_KEYSET
;;               "<channel> :Channel key already set"

(define-public ERR_CHANNELISFULL "471")

;;        471    ERR_CHANNELISFULL
;;               "<channel> :Cannot join channel (+l)"

(define-public ERR_UNKNOWNMODE "472")

;;        472    ERR_UNKNOWNMODE
;;               "<char> :is unknown mode char to me for <channel>"

(define-public ERR_INVITEONLYCHAN "473")

;;        473    ERR_INVITEONLYCHAN
;;               "<channel> :Cannot join channel (+i)"

(define-public ERR_BANNEDFROMCHAN "474")

;;        474    ERR_BANNEDFROMCHAN
;;               "<channel> :Cannot join channel (+b)"

(define-public ERR_BADCHANNELKEY "475")

;;        475    ERR_BADCHANNELKEY
;;               "<channel> :Cannot join channel (+k)"

(define-public ERR_BADCHANMASK "476")

;;        476    ERR_BADCHANMASK
;;               "<channel> :Bad Channel Mask"

(define-public ERR_NOCHANMODES "477")

;;        477    ERR_NOCHANMODES
;;               "<channel> :Channel doesn't support modes"

(define-public ERR_BANLISTFULL "478")

;;        478    ERR_BANLISTFULL
;;               "<channel> <char> :Channel list is full"
;; 

(define-public ERR_NOPRIVILEGES "481")

;;        481    ERR_NOPRIVILEGES
;;               ":Permission Denied- You're not an IRC operator"
;; 
;;          - Any command requiring operator privileges to operate
;;            MUST return this error to indicate the attempt was
;;            unsuccessful.
;; 

(define-public ERR_CHANOPRIVSNEEDED "482")

;;        482    ERR_CHANOPRIVSNEEDED
;;               "<channel> :You're not channel operator"
;; 
;;          - Any command requiring 'chanop' privileges (such as
;;            MODE messages) MUST return this error if the client
;;            making the attempt is not a chanop on the specified
;;            channel.
;; 

(define-public ERR_CANTKILLSERVER "483")

;;        483    ERR_CANTKILLSERVER
;;               ":You can't kill a server!"
;; 
;;          - Any attempts to use the KILL command on a server
;;            are to be refused and this error returned directly
;;            to the client.
;; 

(define-public ERR_RESTRICTED "484")

;;        484    ERR_RESTRICTED
;;               ":Your connection is restricted!"
;; 
;;          - Sent by the server to a user upon connection to indicate
;;            the restricted nature of the connection (user mode "+r").
;; 

(define-public ERR_UNIQOPPRIVSNEEDED "485")

;;        485    ERR_UNIQOPPRIVSNEEDED
;;               ":You're not the original channel operator"
;; 
;;          - Any MODE requiring "channel creator" privileges MUST
;;            return this error if the client making the attempt is not
;;            a chanop on the specified channel.
;; 

(define-public ERR_NOOPERHOST "491")

;;        491    ERR_NOOPERHOST
;;               ":No O-lines for your host"
;; 
;;          - If a client sends an OPER message and the server has
;;            not been configured to allow connections from the
;;            client's host as an operator, this error MUST be
;;            returned.
;; 

(define-public ERR_UMODEUNKNOWNFLAG "501")

;;        501    ERR_UMODEUNKNOWNFLAG
;;               ":Unknown MODE flag"
;; 
;;          - Returned by the server to indicate that a MODE
;;            message was sent with a nickname parameter and that
;;            the a mode flag sent was not recognized.
;; 

(define-public ERR_USERSDONTMATCH "502")

;;        502    ERR_USERSDONTMATCH
;;               ":Cannot change mode for other users"
;; 
;;          - Error sent to any user trying to view or change the
;;            user mode for a user other than themselves.
;; 
